# -*- coding: utf-8 -*-
using ModelingToolkit
using MacroTools
using SemanticModels
using SemanticModels.PetriModels
import MacroTools: postwalk
using Test
using Petri

# +
@variables S, I, R

# generated by AMIDOL, basic SIR Model
sir = model(PetriModel, Petri.Model([S, I, R],
                 [(S+I, 2I), (I,R)]))

# +
@variables I′, R′

# rule1 = SI <- SI -> SII
si = model(PetriModel, Petri.Model([S, I, R],
                 [(S+I, 2I)]))

sii = model(PetriModel, Petri.Model([S, I, I′, R],
                  [(S+I,  2I ),
                   (S+I′, 2I′)]
                 ))

rule1 = PetriModels.PetriSpan(si, si, sii)

# +
# rule2 = I <- I -> IR
i = model(PetriModel, Petri.Model([I′], Tuple{Operation,Operation}[]))

ir = model(PetriModel, Petri.Model([I′, R′], [(I′, R′)]))

rule2 = PetriModels.PetriSpan(i, i, ir)

# +
# rule3 = IRI′R′ <- II′ -> II′R
irir = model(PetriModel, Petri.Model([I, I′, R, R′], [(I, R), (I′, R′)]))

ii = model(PetriModel, Petri.Model([I, I′], Tuple{Operation,Operation}[]))

iir = model(PetriModel, Petri.Model([I, I′, R], [(I, R), (I′, R)]))

rule3 = PetriModels.PetriSpan(irir, ii, iir)

# +
# Apply rules to each model in succession

siir = PetriModels.solve(PetriModels.DPOProblem(rule1, sir))

siirr = PetriModels.solve(PetriModels.DPOProblem(rule2, siir))

c′ = PetriModels.dropdown(irir, ii, siirr)
siir′ = PetriModels.solve(PetriModels.DPOProblem(rule3, c′))

println(sir.model.Δ)
println()
println(siir′.model.Δ)
